@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009, 2012, 2013.
+# Copyright (c) 2009, 2012, 2013, 2014.
 
 # SMHI,
 # Folkborgsvägen 1,
-# Norrköping, 
+# Norrköping,
 # Sweden
 
 # Author(s):
- 
+
 #   Martin Raspaud <martin.raspaud@smhi.se>
 #   Adam Dybbroe <adam.dybbroe@smhi.se>
 #   Esben S. Nielsen <esn@dmi.dk>
@@ -32,10 +32,14 @@
 but has the advandage of using masked arrays as pixel arrays, so that data
 arrays containing invalid values may be properly handled.
 """
+import warnings
+warnings.warn(__name__ + " is deprecated, please use trollimage instead.",
+              DeprecationWarning)
+
 import os
 import re
 
-import Image as Pil
+from PIL import Image as Pil
 import numpy as np
 
 try:
@@ -43,13 +47,19 @@ try:
 except ImportError:
     ne = None
 
-from mpop.imageo.logger import LOG
 from mpop.utils import ensure_dir
 
+import logging
+
+LOG = logging.getLogger(__name__)
+
+
 class UnknownImageFormat(Exception):
+
     """Exception to be raised when image format is unknown to MPOP"""
     pass
 
+
 def check_image_format(fformat):
     cases = {"jpg": "jpeg",
              "jpeg": "jpeg",
@@ -74,7 +84,150 @@ def check_image_format(fformat):
         raise UnknownImageFormat("Unknown image format '%s'." % fformat)
     return fformat
 
+try:
+    from numpy import percentile
+except ImportError:
+
+    # Stolen from numpy 1.7.0 for backward compatibility
+
+    def percentile(a, q, axis=None, out=None, overwrite_input=False):
+        """
+        Compute the qth percentile of the data along the specified axis.
+
+        Returns the qth percentile of the array elements.
+
+        Parameters
+        ----------
+        a : array_like
+        Input array or object that can be converted to an array.
+        q : float in range of [0,100] (or sequence of floats)
+        Percentile to compute which must be between 0 and 100 inclusive.
+        axis : int, optional
+        Axis along which the percentiles are computed. The default (None)
+        is to compute the median along a flattened version of the array.
+        out : ndarray, optional
+        Alternative output array in which to place the result. It must
+        have the same shape and buffer length as the expected output,
+        but the type (of the output) will be cast if necessary.
+        overwrite_input : bool, optional
+        If True, then allow use of memory of input array `a` for
+        calculations. The input array will be modified by the call to
+        median. This will save memory when you do not need to preserve
+        the contents of the input array. Treat the input as undefined,
+        but it will probably be fully or partially sorted.
+        Default is False. Note that, if `overwrite_input` is True and the
+        input is not already an array, an error will be raised.
+
+        Returns
+        -------
+        pcntile : ndarray
+        A new array holding the result (unless `out` is specified, in
+        which case that array is returned instead). If the input contains
+        integers, or floats of smaller precision than 64, then the output
+        data-type is float64. Otherwise, the output data-type is the same
+        as that of the input.
+
+        See Also
+        --------
+        mean, median
+
+        Notes
+        -----
+        Given a vector V of length N, the qth percentile of V is the qth ranked
+        value in a sorted copy of V. A weighted average of the two nearest
+        neighbors is used if the normalized ranking does not match q exactly.
+        The same as the median if ``q=50``, the same as the minimum if ``q=0``
+        and the same as the maximum if ``q=100``.
+
+        Examples
+        --------
+        >>> a = np.array([[10, 7, 4], [3, 2, 1]])
+        >>> a
+        array([[10, 7, 4],
+        [ 3, 2, 1]])
+        >>> np.percentile(a, 50)
+        3.5
+        >>> np.percentile(a, 0.5, axis=0)
+        array([ 6.5, 4.5, 2.5])
+        >>> np.percentile(a, 50, axis=1)
+        array([ 7., 2.])
+
+        >>> m = np.percentile(a, 50, axis=0)
+        >>> out = np.zeros_like(m)
+        >>> np.percentile(a, 50, axis=0, out=m)
+        array([ 6.5, 4.5, 2.5])
+        >>> m
+        array([ 6.5, 4.5, 2.5])
+
+        >>> b = a.copy()
+        >>> np.percentile(b, 50, axis=1, overwrite_input=True)
+        array([ 7., 2.])
+        >>> assert not np.all(a==b)
+        >>> b = a.copy()
+        >>> np.percentile(b, 50, axis=None, overwrite_input=True)
+        3.5
+
+        """
+        a = np.asarray(a)
+
+        if q == 0:
+            return a.min(axis=axis, out=out)
+        elif q == 100:
+            return a.max(axis=axis, out=out)
+
+        if overwrite_input:
+            if axis is None:
+                sorted = a.ravel()
+                sorted.sort()
+            else:
+                a.sort(axis=axis)
+                sorted = a
+        else:
+            sorted = np.sort(a, axis=axis)
+        if axis is None:
+            axis = 0
+
+        return _compute_qth_percentile(sorted, q, axis, out)
+
+    # handle sequence of q's without calling sort multiple times
+    def _compute_qth_percentile(sorted, q, axis, out):
+        if not np.isscalar(q):
+            p = [_compute_qth_percentile(sorted, qi, axis, None)
+                 for qi in q]
+
+            if out is not None:
+                out.flat = p
+
+            return p
+
+        q = q / 100.0
+        if (q < 0) or (q > 1):
+            raise ValueError("percentile must be either in the range [0,100]")
+
+        indexer = [slice(None)] * sorted.ndim
+        Nx = sorted.shape[axis]
+        index = q * (Nx - 1)
+        i = int(index)
+        if i == index:
+            indexer[axis] = slice(i, i + 1)
+            weights = np.array(1)
+            sumval = 1.0
+        else:
+            indexer[axis] = slice(i, i + 2)
+            j = i + 1
+            weights = np.array([(j - index), (index - i)], float)
+            wshape = [1] * sorted.ndim
+            wshape[axis] = 2
+            weights.shape = wshape
+            sumval = weights.sum()
+
+        # Use add.reduce in both cases to coerce data type as well as
+        # check and use out array.
+        return np.add.reduce(sorted[indexer] * weights, axis=axis, out=out) / sumval
+
+
 class Image(object):
+
     """This class defines images. As such, it contains data of the different
     *channels* of the image (red, green, and blue for example). The *mode*
     tells if the channels define a black and white image ("L"), an rgb image
@@ -86,29 +239,28 @@ class Image(object):
     for example will produce black where data is missing."None" (default) will
     produce transparency (thus adding an alpha channel) if the file format
     allows it, black otherwise.
-    
+
     The channels are considered to contain floating point values in the range
     [0.0,1.0]. In order to normalize the input data, the *color_range*
     parameter defines the original range of the data. The conversion to the
     classical [0,255] range and byte type is done automagically when saving the
     image to file.
     """
-    channels = None
-    mode = None
-    width = 0
-    height = 0
-    fill_value = None
-    palette = None
-
-    _secondary_mode = "RGB"
-
     modes = ["L", "LA", "RGB", "RGBA", "YCbCr", "YCbCrA", "P", "PA"]
-    
-    #: Shape (dimensions) of the image.
-    shape = None
-    
-    def __init__(self, channels = None, mode = "L", color_range = None, 
-                 fill_value = None, palette = None):
+
+    def __init__(self, channels=None, mode="L", color_range=None,
+                 fill_value=None, palette=None):
+
+        self.channels = None
+        self.mode = None
+        self.width = 0
+        self.height = 0
+        self.fill_value = None
+        self.palette = None
+        self.shape = None
+        self.info = {}
+
+        self._secondary_mode = "RGB"
 
         if(channels is not None and
            not isinstance(channels, (tuple, set, list,
@@ -131,13 +283,13 @@ class Image(object):
         if(color_range is not None and
            _is_list_of_pairs(color_range) and
            (channels is None or
-            len(color_range) != len(channels))):
+                len(color_range) != len(channels))):
             raise ValueError("Color_range length does not match number of "
                              "channels.")
-        
+
         if(color_range is not None and
            (((mode == "L" or mode == "P") and not _is_pair(color_range)) and
-            (len(color_range) != len(re.findall("[A-Z]", mode))))):
+                (len(color_range) != len(re.findall("[A-Z]", mode))))):
             raise ValueError("Color_range does not match mode")
 
         self.mode = mode
@@ -148,7 +300,7 @@ class Image(object):
             self.fill_value = [fill_value]
         else:
             self.fill_value = None
-            
+
         self.channels = []
         self.palette = palette
 
@@ -162,7 +314,7 @@ class Image(object):
                     else:
                         color_min = 0.0
                         color_max = 1.0
-                    
+
                     # Add data to image object as a channel
                     self._add_channel(chn, color_min, color_max)
 
@@ -185,7 +337,7 @@ class Image(object):
             else:
                 color_min = 0.0
                 color_max = 1.0
-            
+
             # Add data to image object as a channel
             self._add_channel(channels, color_min, color_max)
 
@@ -197,18 +349,18 @@ class Image(object):
     def _add_channel(self, chn, color_min, color_max):
         """Adds a channel to the image object
         """
-        
+
         if isinstance(chn, np.ma.core.MaskedArray):
             chn_data = chn.data
             chn_mask = chn.mask
         else:
             chn_data = np.array(chn)
             chn_mask = False
-        scaled = ((chn_data - color_min) * 
+        scaled = ((chn_data - color_min) *
                   1.0 / (color_max - color_min))
         self.channels.append(np.ma.array(scaled, mask=chn_mask))
-    
-    def _finalize(self):
+
+    def _finalize(self, dtype=np.uint8):
         """Finalize the image, that is put it in RGB mode, and set the channels
         in 8bit format ([0,255] range).
         """
@@ -220,15 +372,16 @@ class Image(object):
 
         for chn in self.channels:
             if isinstance(chn, np.ma.core.MaskedArray):
-                final_data = chn.data.clip(0, 1) * 255
+                final_data = chn.data.clip(0, 1) * np.iinfo(dtype).max
             else:
-                final_data = chn.clip(0, 1) * 255
-                
+                final_data = chn.clip(0, 1) * np.iinfo(dtype).max
+
             channels.append(np.ma.array(final_data,
-                                        np.uint8,
-                                        mask = chn.mask))
+                                        dtype,
+                                        mask=chn.mask))
         if self.fill_value is not None:
-            fill_value = [int(col * 255) for col in self.fill_value]
+            fill_value = [int(col * np.iinfo(dtype).max)
+                          for col in self.fill_value]
         else:
             fill_value = None
         return channels, fill_value
@@ -241,12 +394,10 @@ class Image(object):
             raise RuntimeError("Channels-shape mismatch.")
         return self.channels == [] and self.shape == (0, 0)
 
-
     def show(self):
         """Display the image on screen.
         """
         self.pil_image().show()
-        
 
     def pil_image(self):
         """Return a PIL image from the current image.
@@ -265,7 +416,7 @@ class Image(object):
                 mask = np.ma.getmaskarray(channels[0])
                 alpha = np.where(mask, alpha, 255)
                 pil_alpha = Pil.fromarray(alpha)
-                
+
                 img = Pil.merge("LA", (img, pil_alpha))
         elif(self.mode == "LA"):
             if fill_value is not None:
@@ -282,8 +433,8 @@ class Image(object):
         elif(self.mode == "RGB"):
             # Mask where all channels have missing data (incomplete data will
             # be shown).
-            mask = (np.ma.getmaskarray(channels[0]) & 
-                    np.ma.getmaskarray(channels[1]) & 
+            mask = (np.ma.getmaskarray(channels[0]) &
+                    np.ma.getmaskarray(channels[1]) &
                     np.ma.getmaskarray(channels[2]))
 
             if fill_value is not None:
@@ -305,8 +456,8 @@ class Image(object):
         elif(self.mode == "RGBA"):
             # Mask where all channels have missing data (incomplete data will
             # be shown).
-            mask = (np.ma.getmaskarray(channels[0]) & 
-                    np.ma.getmaskarray(channels[1]) & 
+            mask = (np.ma.getmaskarray(channels[0]) &
+                    np.ma.getmaskarray(channels[1]) &
                     np.ma.getmaskarray(channels[2]) &
                     np.ma.getmaskarray(channels[3]))
 
@@ -327,57 +478,90 @@ class Image(object):
                 img = Pil.merge("RGBA", (pil_r, pil_g, pil_b, pil_a))
 
         else:
-            raise TypeError("Does not know how to use mode %s."%(self.mode))
+            raise TypeError("Does not know how to use mode %s." % (self.mode))
 
         return img
 
-    def save(self, filename, compression = 6, fformat = None):
-        """Save the image to the given *filename*.
+    def save(self, filename, compression=6, fformat=None):
+        """Save the image to the given *filename*. For some formats like jpg
+        and png, the work is delegated to :meth:`pil_save`, which doesn't
+        support the *compression* option.
         """
         self.pil_save(filename, compression, fformat)
 
-    def pil_save(self, filename, compression = 6, fformat = None):
+    def pil_save(self, filename, compression=6, fformat=None):
         """Save the image to the given *filename* using PIL. For now, the
         compression level [0-9] is ignored, due to PIL's lack of support. See
         also :meth:`save`.
         """
         # PIL does not support compression option.
         del compression
-        
+
         if self.is_empty():
             raise IOError("Cannot save an empty image")
-        
+
         ensure_dir(filename)
 
         fformat = fformat or os.path.splitext(filename)[1][1:4]
         fformat = check_image_format(fformat)
 
-        self.pil_image().save(filename, fformat)
-                    
+        params = {}
+
+        if fformat == 'png':
+            # Take care of GeoImage.tags.
+            params['pnginfo'] = self._pngmeta()
+
+        self.pil_image().save(filename, fformat, **params)
+
+    def _pngmeta(self):
+        """It will return GeoImage.tags as a PNG metadata object.
+
+         Inspired by:
+         public domain, Nick Galbreath
+         http://blog.modp.com/2007/08/python-pil-and-png-metadata-take-2.html
+        """
+        reserved = ('interlace', 'gamma', 'dpi', 'transparency', 'aspect')
+
+        try:
+            tags = self.tags
+        except AttributeError:
+            tags = {}
+
+        # Undocumented class
+        from PIL import PngImagePlugin
+        meta = PngImagePlugin.PngInfo()
+
+        # Copy from tags to new dict
+        for k__, v__ in tags.items():
+            if k__ not in reserved:
+                meta.add_text(k__, v__, 0)
+
+        return meta
+
     def putalpha(self, alpha):
         """Adds an *alpha* channel to the current image, or replaces it with
         *alpha* if it already exists.
         """
         alpha = np.ma.array(alpha)
-        if(not (alpha.shape[0] == 0 and self.shape[0] == 0) and
+        if(not (alpha.shape[0] == 0 and
+                self.shape[0] == 0) and
            alpha.shape != self.shape):
             raise ValueError("Alpha channel shape should match image shape")
-        
+
         if(not self.mode.endswith("A")):
-            self.convert(self.mode+"A")
+            self.convert(self.mode + "A")
         if not self.is_empty():
             self.channels[-1] = alpha
-        
+
     def _rgb2ycbcr(self, mode):
         """Convert the image from RGB mode to YCbCr."""
 
         self._check_modes(("RGB", "RGBA"))
 
-
         (self.channels[0], self.channels[1], self.channels[2]) = \
-                          rgb2ycbcr(self.channels[0],
-                                    self.channels[1],
-                                    self.channels[2])
+            rgb2ycbcr(self.channels[0],
+                      self.channels[1],
+                      self.channels[2])
 
         if self.fill_value is not None:
             self.fill_value[0:3] = rgb2ycbcr(self.fill_value[0],
@@ -385,17 +569,17 @@ class Image(object):
                                              self.fill_value[2])
 
         self.mode = mode
-        
+
     def _ycbcr2rgb(self, mode):
         """Convert the image from YCbCr mode to RGB.
-        """ 
+        """
 
         self._check_modes(("YCbCr", "YCbCrA"))
 
         (self.channels[0], self.channels[1], self.channels[2]) = \
-                          ycbcr2rgb(self.channels[0],
-                                    self.channels[1],
-                                    self.channels[2])
+            ycbcr2rgb(self.channels[0],
+                      self.channels[1],
+                      self.channels[2])
 
         if self.fill_value is not None:
             self.fill_value[0:3] = ycbcr2rgb(self.fill_value[0],
@@ -404,7 +588,6 @@ class Image(object):
 
         self.mode = mode
 
-
     def _to_p(self, mode):
         """Convert the image to P or PA mode.
         """
@@ -417,12 +600,12 @@ class Image(object):
             chans = self.channels
             alpha = None
             self._secondary_mode = self.mode
-        
+
         palette = []
         selfmask = reduce(np.ma.mask_or, [chn.mask for chn in chans])
-        new_chn = np.ma.zeros(self.shape, dtype = int)
+        new_chn = np.ma.zeros(self.shape, dtype=int)
         color_nb = 0
-        
+
         for i in range(self.height):
             for j in range(self.width):
                 current_col = tuple([chn[i, j] for chn in chans])
@@ -434,7 +617,7 @@ class Image(object):
                     idx = color_nb
                     palette.append(current_col)
                     color_nb = color_nb + 1
-                    
+
                 new_chn[i, j] = idx
 
         if self.fill_value is not None:
@@ -486,7 +669,7 @@ class Image(object):
             new_chn = np.ma.array(np.interp(color_chan,
                                             np.arange(len(self.palette)),
                                             cdfs[i]),
-                                  mask = color_chan.mask)
+                                  mask=color_chan.mask)
             chans.append(new_chn)
 
         if self.fill_value is not None:
@@ -504,7 +687,6 @@ class Image(object):
             self.mode = self.mode + "A"
 
         self.convert(mode)
-                                        
 
     def _check_modes(self, modes):
         """Check that the image is in on of the given *modes*, raise an
@@ -513,8 +695,7 @@ class Image(object):
         if not isinstance(modes, (tuple, list, set)):
             modes = [modes]
         if self.mode not in modes:
-            raise ValueError("Image not in suitable mode: %s"%modes)
-            
+            raise ValueError("Image not in suitable mode: %s" % modes)
 
     def _l2rgb(self, mode):
         """Convert from L (black and white) to RGB.
@@ -526,7 +707,7 @@ class Image(object):
             self.fill_value = self.fill_value[:1] * 3 + self.fill_value[1:]
         if self.mode == "LA":
             self.channels[1], self.channels[3] = \
-            self.channels[3], self.channels[1]
+                self.channels[3], self.channels[1]
         self.mode = mode
 
     def _rgb2l(self, mode):
@@ -536,11 +717,11 @@ class Image(object):
 
         kb_ = 0.114
         kr_ = 0.299
-        
+
         r__ = self.channels[0]
         g__ = self.channels[1]
         b__ = self.channels[2]
-        
+
         y__ = kr_ * r__ + (1 - kr_ - kb_) * g__ + kb_ * b__
 
         if self.fill_value is not None:
@@ -550,10 +731,9 @@ class Image(object):
                                self.fill_value[3:])
 
         self.channels = [y__] + self.channels[3:]
-        
+
         self.mode = mode
 
-        
     def _ycbcr2l(self, mode):
         """Convert from YCbCr to L.
         """
@@ -568,7 +748,7 @@ class Image(object):
         """Convert from L to YCbCr.
         """
         self._check_modes(("L", "LA"))
-        
+
         luma = self.channels[0]
         zeros = np.ma.zeros(luma.shape)
         zeros.mask = luma.mask
@@ -578,11 +758,8 @@ class Image(object):
         if self.fill_value is not None:
             self.fill_value = [self.fill_value[0], 0, 0] + self.fill_value[1:]
 
-
         self.mode = mode
 
-        
-
     def convert(self, mode):
         """Convert the current image to the given *mode*. See :class:`Image`
         for a list of available modes.
@@ -592,12 +769,12 @@ class Image(object):
 
         if mode not in ["L", "LA", "RGB", "RGBA",
                         "YCbCr", "YCbCrA", "P", "PA"]:
-            raise ValueError("Mode %s not recognized."%(mode))
+            raise ValueError("Mode %s not recognized." % (mode))
 
         if self.is_empty():
             self.mode = mode
             return
-        
+
         if(mode == self.mode + "A"):
             self.channels.append(np.ma.ones(self.channels[0].shape))
             if self.fill_value is not None:
@@ -642,20 +819,20 @@ class Image(object):
                       "YCbCr": self._from_p,
                       "L": self._from_p},
                 "PA": {"RGBA": self._from_p,
-                      "YCbCrA": self._from_p,
-                      "LA": self._from_p}}
+                       "YCbCrA": self._from_p,
+                       "LA": self._from_p}}
             try:
                 cases[self.mode][mode](mode)
             except KeyError:
                 raise ValueError("Conversion from %s to %s not implemented !"
-                                 %(self.mode,mode))
-            
-    def clip(self, channels = True):
+                                 % (self.mode, mode))
+
+    def clip(self, channels=True):
         """Limit the values of the array to the default [0,1] range. *channels*
         says which channels should be clipped."""
         if not (isinstance(channels, (tuple, list))):
-            channels = [channels]*len(self.channels)
-            
+            channels = [channels] * len(self.channels)
+
         for i in range(len(self.channels)):
             if channels[i]:
                 self.channels[i] = np.ma.clip(self.channels[i], 0.0, 1.0)
@@ -682,31 +859,31 @@ class Image(object):
             factor[1] = shape[1] * 1.0 / self.width
         else:
             factor[1] = self.width * 1.0 / shape[1]
-        
+
         if(int(factor[0]) != factor[0] or
            int(factor[1]) != factor[1]):
             raise ValueError("Resize not of integer factor!")
 
         factor[0] = int(factor[0])
         factor[1] = int(factor[1])
-        
+
         i = 0
         for chn in self.channels:
             if zoom[0]:
-                chn = chn.repeat([factor[0]] * chn.shape[0], axis = 0)
+                chn = chn.repeat([factor[0]] * chn.shape[0], axis=0)
             else:
                 chn = chn[[idx * factor[0]
                            for idx in range(self.height / factor[0])],
                           :]
             if zoom[1]:
                 self.channels[i] = chn.repeat([factor[1]] * chn.shape[1],
-                                              axis = 1)
+                                              axis=1)
             else:
                 self.channels[i] = chn[:,
                                        [idx * factor[1]
                                         for idx in range(self.width /
                                                          factor[1])]]
-                
+
             i = i + 1
 
         self.height = self.channels[0].shape[0]
@@ -720,17 +897,17 @@ class Image(object):
         """
         if self.is_empty():
             return
-        
+
         if (luminance.shape != self.channels[0].shape):
             if ((luminance.shape[0] * 1.0 / luminance.shape[1]) ==
-                (self.channels[0].shape[0] * 1.0 / self.channels[0].shape[1])):
+                    (self.channels[0].shape[0] * 1.0 / self.channels[0].shape[1])):
                 if luminance.shape[0] > self.channels[0].shape[0]:
                     self.resize(luminance.shape)
                 else:
                     raise NameError("Luminance smaller than the image !")
             else:
                 raise NameError("Not the good shape !")
-        
+
         mode = self.mode
         if mode.endswith("A"):
             self.convert("YCbCrA")
@@ -740,8 +917,8 @@ class Image(object):
             self.convert("YCbCr")
             self.channels[0] = luminance
             self.convert(mode)
-            
-    def enhance(self, inverse = False, gamma = 1.0, stretch = "no"):
+
+    def enhance(self, inverse=False, gamma=1.0, stretch="no"):
         """Image enhancement function. It applies **in this order** inversion,
         gamma correction, and stretching to the current image, with parameters
         *inverse* (see :meth:`Image.invert`), *gamma* (see
@@ -751,7 +928,7 @@ class Image(object):
         self.gamma(gamma)
         self.stretch(stretch)
 
-    def gamma(self, gamma = 1.0):
+    def gamma(self, gamma=1.0):
         """Apply gamma correction to the channels of the image. If *gamma* is a
         tuple, then it should have as many elements as the channels of the
         image, and the gamma correction is applied elementwise. If *gamma* is a
@@ -768,10 +945,10 @@ class Image(object):
         if(isinstance(gamma, (list, tuple, set)) and
            len(gamma) != len(self.channels)):
             raise ValueError("Number of channels and gamma components differ.")
-        
+
         if gamma < 0:
             raise ValueError("Gamma correction must be a positive number.")
-        
+
         if gamma == 1.0:
             return
         if (isinstance(gamma, (tuple, list))):
@@ -797,8 +974,8 @@ class Image(object):
                                             self.channels[i] **
                                             (1.0 / gamma_list[i]),
                                             self.channels[i])
-        
-    def stretch(self, stretch = "no", **kwarg):
+
+    def stretch(self, stretch="no", **kwarg):
         """Apply stretching to the current image. The value of *stretch* sets
         the type of stretching applied. The values "histogram", "linear",
         "crude" (or "crude-stretch") perform respectively histogram
@@ -810,12 +987,13 @@ class Image(object):
         range [0.0,1.0].
         """
         if((isinstance(stretch, tuple) or
-            isinstance(stretch,list))):
+                isinstance(stretch, list))):
             if len(stretch) == 2:
                 for i in range(len(self.channels)):
-                    self.stretch_linear(i, cutoffs = stretch, **kwarg)
+                    self.stretch_linear(i, cutoffs=stretch, **kwarg)
             else:
-                raise ValueError("Stretch tuple must have exactly two elements")
+                raise ValueError(
+                    "Stretch tuple must have exactly two elements")
         elif stretch == "linear":
             for i in range(len(self.channels)):
                 self.stretch_linear(i, **kwarg)
@@ -831,20 +1009,20 @@ class Image(object):
         elif(stretch == "no"):
             return
         elif isinstance(stretch, str):
-            raise ValueError("Stretching method %s not recognized."%stretch)
+            raise ValueError("Stretching method %s not recognized." % stretch)
         else:
             raise TypeError("Stretch parameter must be a string or a tuple.")
 
-            
-    def invert(self, invert = True):
+    def invert(self, invert=True):
         """Inverts all the channels of a image according to *invert*. If invert
         is a tuple or a list, elementwise invertion is performed, otherwise all
         channels are inverted if *invert* is true (default).
         """
         if(isinstance(invert, (tuple, list, set)) and
            len(self.channels) != len(invert)):
-            raise ValueError("Number of channels and invert components differ.")
-        
+            raise ValueError(
+                "Number of channels and invert components differ.")
+
         if isinstance(invert, (tuple, list, set)):
             i = 0
             for chn in self.channels:
@@ -856,8 +1034,7 @@ class Image(object):
             for chn in self.channels:
                 self.channels[i] = 1.0 - chn
                 i = i + 1
-         
-   
+
     def stretch_hist_equalize(self, ch_nb):
         """Stretch the current image's colors by performing histogram
         equalization on channel *ch_nb*.
@@ -875,17 +1052,17 @@ class Image(object):
 
         carr = arr.compressed()
 
-        imhist, bins = np.histogram(carr, nwidth, normed=True)
-        cdf = imhist.cumsum() - imhist[0]
-        cdf = cdf / cdf[-1]
+        cdf = np.arange(0.0, 1.0, 1 / nwidth)
+        LOG.debug("Make histogram bins having equal amount of data, " +
+                  "using numpy percentile function:")
+        bins = percentile(carr, list(cdf * 100))
 
         res = np.ma.empty_like(arr)
         res.mask = np.ma.getmaskarray(arr)
-        res[~res.mask] = np.interp(carr, bins[:-1], cdf)
+        res[~res.mask] = np.interp(carr, bins, cdf)
 
         self.channels[ch_nb] = res
 
-
     def stretch_logarithmic(self, ch_nb, factor=100.):
         """Move data into range [1:factor] and do a normalized logarithmic
         enhancement.
@@ -893,21 +1070,20 @@ class Image(object):
         LOG.debug("Perform a logarithmic contrast stretch.")
         if ((self.channels[ch_nb].size ==
              np.ma.count_masked(self.channels[ch_nb])) or
-            (self.channels[ch_nb].min() == self.channels[ch_nb].max())):
+                (self.channels[ch_nb].min() == self.channels[ch_nb].max())):
             LOG.warning("Nothing to stretch !")
             return
 
-        crange=(0., 1.0)
+        crange = (0., 1.0)
 
         arr = self.channels[ch_nb]
-        b = float(crange[1] - crange[0])/np.log(factor)
+        b = float(crange[1] - crange[0]) / np.log(factor)
         c = float(crange[0])
-        slope = (factor-1.)/float(arr.max() - arr.min())
-        arr = 1. + (arr - arr.min())*slope
-        arr = c + b*np.log(arr)
+        slope = (factor - 1.) / float(arr.max() - arr.min())
+        arr = 1. + (arr - arr.min()) * slope
+        arr = c + b * np.log(arr)
         self.channels[ch_nb] = arr
 
-
     def stretch_linear(self, ch_nb, cutoffs=(0.005, 0.005)):
         """Stretch linearly the contrast of the current image on channel
         *ch_nb*, using *cutoffs* for left and right trimming.
@@ -919,46 +1095,29 @@ class Image(object):
            self.channels[ch_nb].min() == self.channels[ch_nb].max()):
             LOG.warning("Nothing to stretch !")
             return
-        
-        nwidth = 2048.0
-
 
         arr = self.channels[ch_nb]
-
         carr = arr.compressed()
-        hist, bins = np.histogram(carr, nwidth)
 
-        ndim = carr.size
+        LOG.debug("Calculate the histogram percentiles: ")
+        LOG.debug("Left and right percentiles: " +
+                  str(cutoffs[0] * 100) + " " + str(cutoffs[1] * 100))
 
-        left = 0
-        hist_sum = 0.0
-        i = 0
-        while i < nwidth and hist_sum < cutoffs[0]*ndim:
-            hist_sum = hist_sum + hist[i]
-            i = i + 1
+        left, right = percentile(carr, [cutoffs[0] * 100,
+                                        100. - cutoffs[1] * 100])
 
-        left = bins[i-1]
-
-        right = 0
-        hist_sum = 0.0
-        i = nwidth - 1
-        while i >= 0 and hist_sum < cutoffs[1]*ndim:
-            hist_sum = hist_sum + hist[i]
-            i = i - 1
-
-        right = bins[i+1]
         delta_x = (right - left)
-        LOG.debug("Interval: left=%f,right=%f width=%f"
-                  %(left,right,delta_x))
+        LOG.debug("Interval: left=%f, right=%f width=%f"
+                  % (left, right, delta_x))
 
         if delta_x > 0.0:
-            self.channels[ch_nb] = np.ma.array((arr - left) / delta_x, 
-                                               mask = arr.mask)
+            self.channels[ch_nb] = np.ma.array((arr - left) / delta_x,
+                                               mask=arr.mask)
         else:
             self.channels[ch_nb] = np.ma.zeros(arr.shape)
             LOG.warning("Unable to make a contrast stretch!")
 
-    def crude_stretch(self, ch_nb, min_stretch = None, max_stretch = None):
+    def crude_stretch(self, ch_nb, min_stretch=None, max_stretch=None):
         """Perform simple linear stretching (without any cutoff) on the channel
         *ch_nb* of the current image and normalize to the [0,1] range."""
 
@@ -968,11 +1127,11 @@ class Image(object):
             max_stretch = self.channels[ch_nb].max()
 
         if((not self.channels[ch_nb].mask.all()) and
-            max_stretch - min_stretch > 0):
+                max_stretch - min_stretch > 0):
             stretched = self.channels[ch_nb].data.astype(np.float)
             stretched -= min_stretch
             stretched /= max_stretch - min_stretch
-            self.channels[ch_nb] = np.ma.array(stretched, 
+            self.channels[ch_nb] = np.ma.array(stretched,
                                                mask=self.channels[ch_nb].mask,
                                                copy=False)
         else:
@@ -984,7 +1143,7 @@ class Image(object):
         """
         if self.is_empty():
             raise ValueError("Cannot merge an empty image.")
-        
+
         if(self.mode != img.mode):
             raise ValueError("Cannot merge image of different modes.")
 
@@ -997,16 +1156,19 @@ class Image(object):
             self.channels[i].mask = np.logical_and(selfmask,
                                                    img.channels[i].mask)
 
+
 def all(iterable):
     for element in iterable:
         if not element:
             return False
     return True
 
+
 def _areinstances(the_list, types):
     """Check if all the elements of the list are of given type.
     """
-    return all([isinstance(item, types) for item in the_list]) 
+    return all([isinstance(item, types) for item in the_list])
+
 
 def _is_pair(item):
     """Check if an item is a pair (tuple of size 2).
@@ -1016,6 +1178,7 @@ def _is_pair(item):
             not isinstance(item[0], (list, tuple, set)) and
             not isinstance(item[1], (list, tuple, set)))
 
+
 def _is_list_of_pairs(the_list):
     """Check if a list contains only pairs.
     """
@@ -1024,7 +1187,7 @@ def _is_list_of_pairs(the_list):
 
 def ycbcr2rgb(y__, cb_, cr_):
     """Convert the three YCbCr channels to RGB channels.
-    """ 
+    """
 
     kb_ = 0.114
     kr_ = 0.299
@@ -1035,7 +1198,7 @@ def ycbcr2rgb(y__, cb_, cr_):
 
     return r__, g__, b__
 
-        
+
 def rgb2ycbcr(r__, g__, b__):
     """Convert the three RGB channels to YCbCr."""
 
@@ -1047,4 +1210,3 @@ def rgb2ycbcr(r__, g__, b__):
     cr_ = 1. / (2 * (1 - kr_)) * (r__ - y__)
 
     return y__, cb_, cr_
-
