@@ -137,8 +137,14 @@ def load_generic(satscene, options, calibrate=True, area_extent=None,
             for section in options.keys():
                 if section.endswith('-level1'):
                     break
-            pattern_pro = eval(options[section].get('filename_pro'))
-            pattern_epi = eval(options[section].get('filename_epi'))
+            try:
+                pattern_pro = eval(options[section].get('filename_pro'))
+            except TypeError:
+                pattern_pro = None
+            try:
+                pattern_epi = eval(options[section].get('filename_epi'))
+            except TypeError:
+                pattern_epi = None
             pattern = eval(options[section].get('filename'))
 
             epilogue = None
@@ -147,11 +153,17 @@ def load_generic(satscene, options, calibrate=True, area_extent=None,
 
             if pattern_epi is not None:
                 glob_epi = satscene.time_slot.strftime(
-                    pattern_epi) % ({'segment': "EPI".ljust(9, '_')})
+                    pattern_epi) % ({'segment': "EPI".ljust(9, '_'),
+                                     'channel': chn + '*'})
+            else:
+                glob_epi = 'eggs_and_spam'
 
             if pattern_pro is not None:
                 glob_pro = satscene.time_slot.strftime(
-                    pattern_pro) % ({'segment': "PRO".ljust(9, '_')})
+                    pattern_pro) % ({'segment': "PRO".ljust(9, '_'),
+                                     'channel': chn + '*'})
+            else:
+                glob_pro = 'eggs_and_spam'
 
             glob_img = satscene.time_slot.strftime(
                 pattern) % ({'segment': "*", 'channel': chn + '*'})
@@ -159,9 +171,13 @@ def load_generic(satscene, options, calibrate=True, area_extent=None,
             for filename in filenames:
                 if fnmatch.fnmatch(os.path.basename(filename), glob_img):
                     image_files.append(filename)
-                elif fnmatch.fnmatch(os.path.basename(filename), glob_pro):
+                elif pattern_pro is not None and fnmatch.fnmatch(
+                    os.path.basename(filename),
+                        glob_pro):
                     prologue = filename
-                elif fnmatch.fnmatch(os.path.basename(filename), glob_epi):
+                elif pattern_epi is not None and fnmatch.fnmatch(
+                    os.path.basename(filename),
+                        glob_epi):
                     epilogue = filename
             if len(image_files) == 0 and prologue is None and epilogue is None:
                 use_filenames = False
